from upgradeChat.models.order import Model as Order
from upgradeChat.models.orders import Model as Orders
from upgradeChat.models.product import Model as Product
from upgradeChat.models.products import Model as Products
from upgradeChat.models.user import Model as User
from upgradeChat.models.users import Model as Users
from upgradeChat.models.webhook import Model as Webhook
from upgradeChat.models.webhooks import Model as Webhooks
from upgradeChat.models.webhook_event import Model as WebhookEvent
from upgradeChat.models.webhook_events import Model as WebookEvents