# generated by datamodel-codegen:
#   filename:  model.json
#   timestamp: 2021-04-11T02:34:32+00:00

from __future__ import annotations

from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel, Field


class User(BaseModel):
    discord_id: str
    username: str


class Model(BaseModel):
    data: User

    def __str__(self):
        return f"<User {self.data.username}>"
