from upgradeChat.api import API

__version__ = '0.1.1'

__title__ = "upgradeChat"
__github_repo__ = "upgradeChat"
__author__ = 'Nate Harris'
__author_email__ = 'n8gr8gbln@gmail.com'
__github_username__ = "nwithan8"
__copyright__ = "Copyright © 2021 - Nate Harris"
__license__ = 'GNU General Public License v3 (GPLv3)'
__description__ = "A complete Python client for Upgrade.Chat's API"
__keywords__ = ["Discord", "chat", "bot", "Upgrade.Chat", "donations", "subscriptions", "rewards", "management", "API", "client"]